@@ -34,7 +34,7 @@ our @EXPORT_OK =
   qw(BY_XPATH BY_ID BY_NAME BY_TAG BY_CLASS BY_SELECTOR BY_LINK BY_PARTIAL);
 our %EXPORT_TAGS = ( all => \@EXPORT_OK );
 
-our $VERSION = '0.38';
+our $VERSION = '0.40';
 
 sub _ANYPROCESS                    { return -1 }
 sub _COMMAND                       { return 0 }
@@ -50,14 +50,61 @@ sub _MIN_VERSION_FOR_WD_HEADLESS   { return 56 }
 sub _MIN_VERSION_FOR_SAFE_MODE     { return 55 }
 sub _MIN_VERSION_FOR_AUTO_LISTEN   { return 55 }
 
-sub BY_XPATH    { return 'xpath' }
-sub BY_ID       { return 'id' }
-sub BY_NAME     { return 'name' }
-sub BY_TAG      { return 'tag name' }
-sub BY_CLASS    { return 'class name' }
-sub BY_SELECTOR { return 'css selector' }
-sub BY_LINK     { return 'link text' }
-sub BY_PARTIAL  { return 'partial link text' }
+sub BY_XPATH {
+    Carp::carp(
+'**** DEPRECATED METHOD - using find(..., BY_XPATH()) HAS BEEN REPLACED BY find ****'
+    );
+    return 'xpath';
+}
+
+sub BY_ID {
+    Carp::carp(
+'**** DEPRECATED METHOD - using find(..., BY_ID()) HAS BEEN REPLACED BY find_by_id ****'
+    );
+    return 'id';
+}
+
+sub BY_NAME {
+    Carp::carp(
+'**** DEPRECATED METHOD - using find(..., BY_NAME()) HAS BEEN REPLACED BY find_by_name ****'
+    );
+    return 'name';
+}
+
+sub BY_TAG {
+    Carp::carp(
+'**** DEPRECATED METHOD - using find(..., BY_TAG()) HAS BEEN REPLACED BY find_by_tag ****'
+    );
+    return 'tag name';
+}
+
+sub BY_CLASS {
+    Carp::carp(
+'**** DEPRECATED METHOD - using find(..., BY_CLASS()) HAS BEEN REPLACED BY find_by_class ****'
+    );
+    return 'class name';
+}
+
+sub BY_SELECTOR {
+    Carp::carp(
+'**** DEPRECATED METHOD - using find(..., BY_SELECTOR()) HAS BEEN REPLACED BY find_by_selector ****'
+    );
+    return 'css selector';
+}
+
+sub BY_LINK {
+    Carp::carp(
+'**** DEPRECATED METHOD - using find(..., BY_LINK()) HAS BEEN REPLACED BY find_by_link ****'
+    );
+    return 'link text';
+}
+
+sub BY_PARTIAL {
+    Carp::carp(
+'**** DEPRECATED METHOD - using find(..., BY_PARTIAL()) HAS BEEN REPLACED BY find_by_partial ****'
+    );
+    return 'partial link text';
+}
 
 sub new {
     my ( $class, %parameters ) = @_;
@@ -808,9 +855,7 @@ sub _get_port {
           or Carp::croak(
             "Failed to close '$self->{profile_path}':$EXTENDED_OS_ERROR");
     }
-    if ($port) {
-        return $port;
-    }
+    return $port;
 }
 
 sub _initial_socket_setup {
@@ -936,27 +981,71 @@ sub _create_capabilities {
 sub find_elements {
     my ( $self, $value, $using ) = @_;
     Carp::carp(
-        '**** DEPRECATED METHOD - find_elements HAS BEEN REPLACED BY list ****'
+        '**** DEPRECATED METHOD - find_elements HAS BEEN REPLACED BY find ****'
     );
-    return $self->find( $value, $using );
+    return $self->_find( $value, $using );
 }
 
 sub list {
-    my ( $self, $value, $using ) = @_;
-    $using ||= BY_XPATH();
-    my $message_id = $self->_new_message_id();
-    $self->_send_request(
-        [
-            _COMMAND(),
-            $message_id,
-            $self->_command('WebDriver:FindElements'),
-            { using => $using, value => $value }
-        ]
+    my ( $self, $value, $using, $from ) = @_;
+    Carp::carp('**** DEPRECATED METHOD - list HAS BEEN REPLACED BY find ****');
+    return $self->_find( $value, $using, $from );
+}
+
+sub list_by_id {
+    my ( $self, $value, $from ) = @_;
+    Carp::carp(
+'**** DEPRECATED METHOD - list_by_id HAS BEEN REPLACED BY find_by_id ****'
     );
-    my $response = $self->_get_response($message_id);
-    return
-      map { Firefox::Marionette::Element->new( $self, %{$_} ) }
-      @{ $response->result() };
+    return $self->_find( $value, 'id', $from );
+}
+
+sub list_by_name {
+    my ( $self, $value, $from ) = @_;
+    Carp::carp(
+'**** DEPRECATED METHOD - list_by_name HAS BEEN REPLACED BY find_by_name ****'
+    );
+    return $self->_find( $value, 'name', $from );
+}
+
+sub list_by_tag {
+    my ( $self, $value, $from ) = @_;
+    Carp::carp(
+'**** DEPRECATED METHOD - list_by_tag HAS BEEN REPLACED BY find_by_tag ****'
+    );
+    return $self->_find( $value, 'tag name', $from );
+}
+
+sub list_by_class {
+    my ( $self, $value, $from ) = @_;
+    Carp::carp(
+'**** DEPRECATED METHOD - list_by_class HAS BEEN REPLACED BY find_by_class ****'
+    );
+    return $self->_find( $value, 'class name', $from );
+}
+
+sub list_by_selector {
+    my ( $self, $value, $from ) = @_;
+    Carp::carp(
+'**** DEPRECATED METHOD - list_by_selector HAS BEEN REPLACED BY find_by_selector ****'
+    );
+    return $self->_find( $value, 'css selector', $from );
+}
+
+sub list_by_link {
+    my ( $self, $value, $from ) = @_;
+    Carp::carp(
+'**** DEPRECATED METHOD - list_by_link HAS BEEN REPLACED BY find_by_link ****'
+    );
+    return $self->_find( $value, 'link text', $from );
+}
+
+sub list_by_partial {
+    my ( $self, $value, $from ) = @_;
+    Carp::carp(
+'**** DEPRECATED METHOD - list_by_partial HAS BEEN REPLACED BY find_by_partial ****'
+    );
+    return $self->_find( $value, 'partial link text', $from );
 }
 
 sub add_cookie {
@@ -1756,20 +1845,67 @@ sub find_element {
 }
 
 sub find {
-    my ( $self, $value, $using ) = @_;
-    $using ||= BY_XPATH();
+    my ( $self, $value, $using, $from ) = @_;
+    return $self->_find( $value, $using, $from );
+}
+
+sub find_by_id {
+    my ( $self, $value, $from ) = @_;
+    return $self->_find( $value, 'id', $from );
+}
+
+sub find_by_name {
+    my ( $self, $value, $from ) = @_;
+    return $self->_find( $value, 'name', $from );
+}
+
+sub find_by_tag {
+    my ( $self, $value, $from ) = @_;
+    return $self->_find( $value, 'tag name', $from );
+}
+
+sub find_by_class {
+    my ( $self, $value, $from ) = @_;
+    return $self->_find( $value, 'class name', $from );
+}
+
+sub find_by_selector {
+    my ( $self, $value, $from ) = @_;
+    return $self->_find( $value, 'css selector', $from );
+}
+
+sub find_by_link {
+    my ( $self, $value, $from ) = @_;
+    return $self->_find( $value, 'link text', $from );
+}
+
+sub find_by_partial {
+    my ( $self, $value, $from ) = @_;
+    return $self->_find( $value, 'partial link text', $from );
+}
+
+sub _find {
+    my ( $self, $value, $using, $from ) = @_;
+    $using ||= 'xpath';
     my $message_id = $self->_new_message_id();
+    my $parameters = { using => $using, value => $value };
+    if ( defined $from ) {
+        $parameters->{element} = $from->uuid();
+    }
+    my $command =
+      wantarray ? 'WebDriver:FindElements' : 'WebDriver:FindElement';
     $self->_send_request(
-        [
-            _COMMAND(),
-            $message_id,
-            $self->_command('WebDriver:FindElement'),
-            { using => $using, value => $value }
-        ]
-    );
+        [ _COMMAND(), $message_id, $self->_command($command), $parameters, ] );
     my $response = $self->_get_response($message_id);
-    return Firefox::Marionette::Element->new( $self,
-        %{ $response->result()->{value} } );
+    if (wantarray) {
+        return
+          map { Firefox::Marionette::Element->new( $self, %{$_} ) }
+          @{ $response->result() };
+    }
+    else {
+        return Firefox::Marionette::Element->new( $self,
+            %{ $response->result()->{value} } );
+    }
 }
 
 sub active_frame {
@@ -2287,18 +2423,18 @@ Firefox::Marionette - Automate the Firefox browser with the Marionette protocol
 
 =head1 VERSION
 
-Version 0.38
+Version 0.40
 
 =head1 SYNOPSIS
 
-    use Firefox::Marionette qw(:all);
+    use Firefox::Marionette();
     use v5.10;
 
     my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
 
-    $firefox->find('search-input', BY_ID())->type('Test::More');
+    $firefox->find_by_class('container-fluid')->find_by_id('search-input')->type('Test::More');
 
-    my $file_handle = $firefox->selfie(highlights => [ $firefox->find('lucky', BY_NAME()) ])
+    my $file_handle = $firefox->selfie(highlights => [ $firefox->find_by_name('lucky') ])
 
     $firefox->find('//button[@name="lucky"]')->click();
 
@@ -2348,62 +2484,152 @@ returns the current L<URI|URI> of current top level browsing context for Desktop
 
 returns the current title of the window.
 
-=head2 find_element
-
-*** DEPRECATED - see L<find|Firefox::Marionette#find>. ***
-
 =head2 find
 
-returns the first element in the current browsing context that matches the search parameters supplied;
-
-=over 4
-
-=item * the first parameter is a scalar search value.  This can be an L<xpath|https://en.wikipedia.org/wiki/XPath> expression such as C<//button[@name="foo"]> to find all button elements that have a 'name' of 'foo'.
-
-=item * the second optional parameter is a scalar search strategy.  This defaults to L<BY_XPATH()|Firefox::Marionette#BY_XPATH>
-
-=back
+accepts an L<xpath expression|https://en.wikipedia.org/wiki/XPath> expression> as the first parameter and returns the first L<element|Firefox::Marionette::Element> that matches this expression.
 
 This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout.
 
-    use Firefox::Marionette qw(:all);
+    use Firefox::Marionette();
     use v5.10;
 
     my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
 
-    # All these methods accomplish the same goal
-
     $firefox->find('//input[@id="search-input"]')->type('Test::More');
-    $firefox->find('//input[@id="search-input"]', BY_XPATH())->type('Test::More');
-    $firefox->find('search-input', BY_ID())->type('Test::More');
-    $firefox->find('q', BY_NAME())->type('Test::More');
-    $firefox->find('form-control home-search-input', BY_CLASS())->type('Test::More');
-    $firefox->find('input.home-search-input', BY_SELECTOR())->type('Test::More');
 
-    # Some additional strategies
+    # OR in list context 
 
-    $firefox->find('input', BY_TAG());
-    $firefox->find('API', BY_LINK());
-    $firefox->find('AP', BY_PARTIAL());
+    foreach my $element ($firefox->find('//input[@id="search-input"]')) {
+        $element->type('Test::More');
+    }
 
-=head2 find_elements
+=head2 find_by_id
 
-*** DEPRECATED - see L<list|Firefox::Marionette#list>. ***
-
-=head2 list
-
-returns all the elements in the current browsing context that match the search parameters supplied;
-
-=over 4
-
-=item * the first parameter is a scalar search value.  This can be an L<xpath|https://en.wikipedia.org/wiki/XPath> expression such as C<//button[@name="foo"]> to find all button elements that have a 'name' of 'foo'.
-
-=item * the second optional parameter is a scalar search strategy.  This defaults to L<BY_XPATH()|Firefox::Marionette#BY_XPATH>.
-
-=back
+accepts an L<id|https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/id> as the first parameter and returns the first L<element|Firefox::Marionette::Element> with a matching 'id' property.
 
 This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout.
 
+    use Firefox::Marionette();
+    use v5.10;
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+
+    $firefox->find_by_id('search-input')->type('Test::More');
+
+    # OR in list context 
+
+    foreach my $element ($firefox->find_by_id('search-input')) {
+        $element->type('Test::More');
+    }
+
+=head2 find_by_name
+
+This method returns the first L<element|Firefox::Marionette::Element> with a matching 'name' property.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout.
+
+    use Firefox::Marionette();
+    use v5.10;
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+    $firefox->find_by_name('q')->type('Test::More');
+
+    # OR in list context 
+
+    foreach my $element ($firefox->find_by_name('q')) {
+        $element->type('Test::More');
+    }
+
+=head2 find_by_class
+
+accepts a L<class name|https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/class> as the first parameter and returns the first L<element|Firefox::Marionette::Element> with a matching 'class' property.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout.
+
+    use Firefox::Marionette();
+    use v5.10;
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+    $firefox->find_by_class('form-control home-search-input')->type('Test::More');
+
+    # OR in list context 
+
+    foreach my $element ($firefox->find_by_class('form-control home-search-input')) {
+        $element->type('Test::More');
+    }
+
+=head2 find_by_selector
+
+accepts a L<CSS Selector|https://developer.mozilla.org/en-US/docs/Web/CSS/CSS_Selectors> as the first parameter and returns the first L<element|Firefox::Marionette::Element> that matches that selector.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout.
+
+    use Firefox::Marionette();
+    use v5.10;
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+    $firefox->find_by_selector('input.home-search-input')->type('Test::More');
+
+    # OR in list context 
+
+    foreach my $element ($firefox->list_by_selector('input.home-search-input')) {
+        $element->type('Test::More');
+    }
+
+=head2 find_by_tag
+
+accepts a L<tag name|https://developer.mozilla.org/en-US/docs/Web/API/Element/tagName> as the first parameter and returns the first L<element|Firefox::Marionette::Element> with this tag name.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout.
+
+    use Firefox::Marionette();
+    use v5.10;
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+    my $element = $firefox->find_by_tag('input');
+
+    # OR in list context 
+
+    foreach my $element ($firefox->list_by_tag('input')) {
+        # do something
+    }
+
+=head2 find_by_link
+
+accepts a text string as the first parameter and returns the first link L<element|Firefox::Marionette::Element> that has a matching link text.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout.
+
+    use Firefox::Marionette();
+    use v5.10;
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+    $firefox->find_by_link('API')->click();
+
+    # OR in list context 
+
+    foreach my $element ($firefox->list_by_link('API')) {
+        $element->click();
+    }
+
+=head2 find_by_partial
+
+accepts a text string as the first parameter and returns the first link L<element|Firefox::Marionette::Element> that has a partially matching link text.
+
+This method is subject to the L<implicit|Firefox::Marionette::Timeouts#implicit> timeout.
+
+    use Firefox::Marionette();
+    use v5.10;
+
+    my $firefox = Firefox::Marionette->new()->go('https://metacpan.org/');
+    $firefox->find_by_partial('AP')->click();
+
+    # OR in list context 
+
+    foreach my $element ($firefox->list_by_partial('AP')) {
+        $element->click();
+    }
+
 =head2 css
 
 accepts an L<element|Firefox::Marionette::Element> as the first parameter and a scalar CSS property name as the second parameter.  It returns the value of the computed style for that property.
@@ -2428,10 +2654,6 @@ accepts a scalar containing a javascript function that is executed in the browse
 
 The executing javascript is subject to the L<scripts|Firefox::Marionette::Timeouts#scripts> timeout.
 
-=head2 page_source
-
-*** DEPRECATED - see L<html|Firefox::Marionette#html>. ***
-
 =head2 html
 
 returns the page source of the content document.
@@ -2456,10 +2678,6 @@ here be cookie monsters! This method returns L<itself|Firefox::Marionette> to ai
 
 returns the contents of the cookie jar in scalar or list context.
 
-=head2 send_keys
-
-*** DEPRECATED - see L<type|Firefox::Marionette#type>. ***
-
 =head2 type
 
 accepts an L<element|Firefox::Marionette::Element> as the first parameter and a string as the second parameter.  It sends the string to the specified L<element|Firefox::Marionette::Element> in the current page, such as filling out a text box. This method returns L<itself|Firefox::Marionette> to aid in chaining methods.
@@ -2688,38 +2906,6 @@ This method returns a human readable error message describing how the Firefox pr
 
 This method returns true or false depending on if the Firefox process is still running.
 
-=head2 BY_XPATH
-
-This L<find|Firefox::Marionette#find> strategy returns the element(s) that match with supplied L<xpath|https://en.wikipedia.org/wiki/XPath> expression.
-
-=head2 BY_ID
-
-This L<find|Firefox::Marionette#find> strategy returns the element(s) with a matching 'id' property.
-
-=head2 BY_NAME
-
-This L<find|Firefox::Marionette#find> strategy returns the element(s) with a matching 'name' property.
-
-=head2 BY_CLASS
-
-This L<find|Firefox::Marionette#find> strategy returns the element(s) with a matching 'class' property.
-
-=head2 BY_SELECTOR
-
-This L<find|Firefox::Marionette#find> strategy returns the element(s) with a matching L<CSS Selector|https://en.wikipedia.org/wiki/Cascading_Style_Sheets#Selector>
-
-=head2 BY_TAG
-
-This L<find|Firefox::Marionette#find> strategy returns the element(s) with a matching tag name.
-
-=head2 BY_LINK
-
-This L<find|Firefox::Marionette#find> strategy returns the link(s) with matching text
-
-=head2 BY_PARTIAL
-
-This L<find|Firefox::Marionette#find> strategy returns the link(s) with matching partial text
-
 =head1 DIAGNOSTICS
 
 =over
@@ -2749,6 +2935,9 @@ Firefox::Marionette requires the following non-core Perl modules
  
 =over
  
+=item *
+L<IPC::Run|IPC::Run>
+ 
 =item *
 L<JSON|JSON>
  
